@@ -1,11 +1,14 @@
 import pickle
 import pytest
 
-@pytest.fixture(scope='session', params=['pyrsistent', 'pvectorc'])
+from pyrsistent._pvector import python_pvector
+
+
+@pytest.fixture(scope='session', params=['pyrsistent._pvector', 'pvectorc'])
 def pvector(request):
     m = pytest.importorskip(request.param)
-    if request.param == 'pyrsistent':
-        return m._pvector
+    if request.param == 'pyrsistent._pvector':
+        return m.python_pvector
     return m.pvector
 
 
@@ -19,8 +22,9 @@ def test_empty_initialization(pvector):
     seq = pvector()
     assert len(seq) == 0
 
-    with pytest.raises(IndexError):
+    with pytest.raises(IndexError) as error:
         x = seq[0]
+    assert str(error.value) == 'Index out of range: 0'
 
 
 def test_initialization_with_one_element(pvector):
@@ -111,9 +115,11 @@ def test_insert_beyond_end(pvector):
     seq2 = seq.set(2, 50)
     assert seq2[2] == 50
 
-    with pytest.raises(IndexError):
+    with pytest.raises(IndexError) as error:
         seq2.set(19, 4)
 
+    assert str(error.value) == 'Index out of range: 19'
+
 
 def test_insert_with_index_from_the_end(pvector):
     x = pvector([1, 2, 3, 4])
@@ -228,26 +234,61 @@ def test_slicing_reverse(pvector):
     assert len(seq3) == 4
 
 
+def test_delete_index(pvector):
+    seq = pvector([1, 2, 3])
+    assert seq.delete(0) == pvector([2, 3])
+    assert seq.delete(1) == pvector([1, 3])
+    assert seq.delete(2) == pvector([1, 2])
+    assert seq.delete(-1) == pvector([1, 2])
+    assert seq.delete(-2) == pvector([1, 3])
+    assert seq.delete(-3) == pvector([2, 3])
+
+
+def test_delete_index_out_of_bounds(pvector):
+    with pytest.raises(IndexError):
+        pvector([]).delete(0)
+    with pytest.raises(IndexError):
+        pvector([]).delete(-1)
+
+
+def test_delete_index_malformed(pvector):
+    with pytest.raises(TypeError):
+        pvector([]).delete('a')
+
+
+def test_delete_slice(pvector):
+    seq = pvector(range(5))
+    assert seq.delete(1, 4) == pvector([0, 4])
+    assert seq.delete(4, 1) == seq
+    assert seq.delete(0, 1) == pvector([1, 2, 3, 4])
+    assert seq.delete(6, 8) == seq
+    assert seq.delete(-1, 1) == seq
+    assert seq.delete(1, -1) == pvector([0, 4])
+
+
+def test_remove(pvector):
+    seq = pvector(range(5))
+    assert seq.remove(3) == pvector([0, 1, 2, 4])
+
+
+def test_remove_first_only(pvector):
+    seq = pvector([1, 2, 3, 2, 1])
+    assert seq.remove(2) == pvector([1, 3, 2, 1])
+
+
+def test_remove_index_out_of_bounds(pvector):
+    seq = pvector(range(5))
+    with pytest.raises(ValueError) as err:
+        seq.remove(5)
+    assert 'not in' in str(err.value)
+
+
 def test_addition(pvector):
     v = pvector([1, 2]) + pvector([3, 4])
 
     assert list(v) == [1, 2, 3, 4]
 
 
-def test_slicing_reverse(pvector):
-    seq = pvector(range(10))
-    seq2 = seq[::-1]
-
-    assert seq2[0] == 9
-    assert seq2[1] == 8
-    assert len(seq2) == 10
-
-    seq3 = seq[-3: -7: -1]
-    assert seq3[0] == 7
-    assert seq3[3] == 4
-    assert len(seq3) == 4
-
-
 def test_sorted(pvector):
     seq = pvector([5, 2, 3, 1])
     assert [1, 2, 3, 5] == sorted(seq)
@@ -366,50 +407,50 @@ def test_repeat(pvector):
     assert -3 * pvector([1, 2]) is pvector()
 
 
-def test_set_zero_key_length(pvector):
+def test_transform_zero_key_length(pvector):
     x = pvector([1, 2])
 
-    assert x.set_in([], 3) is x
+    assert x.transform([], 3) == 3
 
 
-def test_set_in_base_case(pvector):
+def test_transform_base_case(pvector):
     x = pvector([1, 2])
 
-    assert x.set_in([1], 3) == pvector([1, 3])
+    assert x.transform([1], 3) == pvector([1, 3])
 
 
-def test_set_in_nested_vectors(pvector):
+def test_transform_nested_vectors(pvector):
     x = pvector([1, 2, pvector([3, 4]), 5])
 
-    assert x.set_in([2, 0], 999) == pvector([1, 2, pvector([999, 4]), 5])
+    assert x.transform([2, 0], 999) == pvector([1, 2, pvector([999, 4]), 5])
 
 
-def test_set_in_when_appending(pvector):
+def test_transform_when_appending(pvector):
     from pyrsistent import m
     x = pvector([1, 2])
 
-    assert x.set_in([2, 'd'], 999) == pvector([1, 2, m(d=999)])
+    assert x.transform([2, 'd'], 999) == pvector([1, 2, m(d=999)])
 
 
-def test_set_in_index_error_out_range(pvector):
+def test_transform_index_error_out_range(pvector):
     x = pvector([1, 2, pvector([3, 4]), 5])
 
     with pytest.raises(IndexError):
-        x.set_in([2, 10], 999)
+        x.transform([2, 10], 999)
 
 
-def test_set_in_index_error_wrong_type(pvector):
+def test_transform_index_error_wrong_type(pvector):
     x = pvector([1, 2, pvector([3, 4]), 5])
 
     with pytest.raises(TypeError):
-        x.set_in([2, 'foo'], 999)
+        x.transform([2, 'foo'], 999)
 
 
-def test_set_in_non_setable_type(pvector):
+def test_transform_non_setable_type(pvector):
     x = pvector([1, 2, 5])
 
-    with pytest.raises(AttributeError):
-        x.set_in([2, 3], 999)
+    with pytest.raises(TypeError):
+        x.transform([2, 3], 999)
 
 
 def test_reverse(pvector):
@@ -498,7 +539,7 @@ def test_evolver_no_update(pvector):
     # This is mostly a test against memory leaks in the C implementation
     v = pvector(range(40))
 
-    assert v.evolver().pvector() == v
+    assert v.evolver().persistent() == v
 
 
 def test_evolver_deallocate_dirty_evolver(pvector):
@@ -515,9 +556,16 @@ def test_evolver_simple_update_in_tree(pvector):
     e[10] = -10
 
     assert e[10] == -10
-    assert e.pvector()[10] == -10
+    assert e.persistent()[10] == -10
 
 
+def test_evolver_set_out_of_range(pvector):
+    v = pvector([0])
+    e = v.evolver()
+    with pytest.raises(IndexError) as error:
+        e[10] = 1
+    assert str(error.value) == "Index out of range: 10"
+
 def test_evolver_multi_level_multi_update_in_tree(pvector):
     # This test is mostly to detect memory/ref count issues in the native implementation
     v = pvector(range(3500))
@@ -547,7 +595,7 @@ def test_evolver_multi_level_multi_update_in_tree(pvector):
     import gc
     gc.collect()
 
-    v2 = e.pvector()
+    v2 = e.persistent()
     assert v2[10] == -1000
     assert v2[50] == -5000
     assert v2[3000] == -30000
@@ -573,7 +621,7 @@ def test_evolver_simple_update_in_tail(pvector):
     e[33] = -33
 
     assert e[33] == -33
-    assert e.pvector()[33] == -33
+    assert e.persistent()[33] == -33
     assert v[33] == 33
 
 
@@ -583,7 +631,7 @@ def test_evolver_simple_update_just_outside_vector(pvector):
     e[0] = 1
 
     assert e[0] == 1
-    assert e.pvector()[0] == 1
+    assert e.persistent()[0] == 1
     assert len(v) == 0
 
 
@@ -595,7 +643,7 @@ def test_evolver_append(pvector):
 
     e[0] = 2000
     assert e[0] == 2000
-    assert list(e.pvector()) == [2000]
+    assert list(e.persistent()) == [2000]
     assert list(v) == []
 
 
@@ -605,7 +653,7 @@ def test_evolver_extend(pvector):
     e.extend([2000, 3000])
     e[2] = 20000
 
-    assert list(e.pvector()) == [1000, 2000, 20000]
+    assert list(e.persistent()) == [1000, 2000, 20000]
     assert list(v) == [1000]
 
 
@@ -617,7 +665,7 @@ def test_evolver_assign_and_read_with_negative_indices(pvector):
     e[-1] = 33
 
     assert e[-1] == 33
-    assert list(e.pvector()) == [1, 2, 4, 11, 12, 33]
+    assert list(e.persistent()) == [1, 2, 4, 11, 12, 33]
 
 
 def test_evolver_non_integral_access(pvector):
@@ -665,12 +713,12 @@ def test_no_dependencies_between_evolvers_from_the_same_pvector(pvector):
     e1_expected = original_list + [1, 2, 3]
     e1_expected[2] = 20
     e1_expected[35] = 350
-    assert list(e1.pvector()) == e1_expected
+    assert list(e1.persistent()) == e1_expected
 
     e2_expected = original_list + [-1, -2, -3]
     e2_expected[2] = -20
     e2_expected[35] = -350
-    assert list(e2.pvector()) == e2_expected
+    assert list(e2.persistent()) == e2_expected
 
 
 def test_pvectors_produced_from_the_same_evolver_do_not_interfere(pvector):
@@ -682,7 +730,7 @@ def test_pvectors_produced_from_the_same_evolver_do_not_interfere(pvector):
     e[2] = 20
     e[35] = 350
 
-    v1 = e.pvector()
+    v1 = e.persistent()
     v1_expected = original_list + [1, 2, 3]
     v1_expected[2] = 20
     v1_expected[35] = 350
@@ -691,7 +739,7 @@ def test_pvectors_produced_from_the_same_evolver_do_not_interfere(pvector):
     e[3] = -30
     e[36] = -360
 
-    v2 = e.pvector()
+    v2 = e.persistent()
     v2_expected = v1_expected + [-1, -2, -3]
     v2_expected[3] = -30
     v2_expected[36] = -360
@@ -714,11 +762,144 @@ def test_evolver_is_dirty(pvector):
     e.append(4)
     assert e.is_dirty
 
-    e.pvector()
+    e.persistent()
     assert not e.is_dirty()
 
     e[2] = 2000
     assert e.is_dirty
 
-    e.pvector()
+    e.persistent()
     assert not e.is_dirty()
+
+
+def test_vector_insert_one_step_beyond_end(pvector):
+    # This test exists to get the transform functionality under memory
+    # leak supervision. Most of the transformation tests are in test_transform.py.
+    v = pvector([1, 2])
+    assert v.transform([2], 3) == pvector([1, 2, 3])
+
+
+def test_evolver_with_no_updates_returns_same_pvector(pvector):
+    v = pvector([1, 2])
+    assert v.evolver().persistent() is v
+
+
+def test_evolver_returns_itself_on_evolving_operations(pvector):
+    # Does this to be able to chain operations
+    v = pvector([1, 2])
+    assert v.evolver().append(3).extend([4, 5]).set(1, 6).persistent() == pvector([1, 6, 3, 4, 5])
+
+
+def test_evolver_delete_by_index(pvector):
+    e = pvector([1, 2, 3]).evolver()
+
+    del e[0]
+
+    assert e.persistent() == python_pvector([2, 3])
+    assert e.append(4).persistent() == python_pvector([2, 3, 4])
+
+
+def test_evolver_delete_function_by_index(pvector):
+    e = pvector([1, 2, 3]).evolver()
+
+    assert e.delete(1).persistent() == python_pvector([1, 3])
+
+
+def test_evolver_delete_function_by_index_multiple_times(pvector):
+    SIZE = 40
+    e = pvector(range(SIZE)).evolver()
+    for i in range(SIZE):
+        assert e[0] == i
+        assert list(e.persistent()) == list(range(i, SIZE))
+        del e[0]
+
+    assert e.persistent() == list()
+
+
+def test_evolver_delete_function_invalid_index(pvector):
+    e = pvector([1, 2]).evolver()
+
+    with pytest.raises(TypeError):
+        del e["e"]
+
+
+def test_delete_of_non_existing_element(pvector):
+    e = pvector([1, 2]).evolver()
+
+    with pytest.raises(IndexError):
+        del e[2]
+
+    del e[0]
+    del e[0]
+
+    with pytest.raises(IndexError):
+        del e[0]
+
+    assert e.persistent() == pvector()
+
+
+def test_append_followed_by_delete(pvector):
+    e = pvector([1, 2]).evolver()
+
+    e.append(3)
+
+    del e[2]
+
+
+def test_evolver_set_followed_by_delete(pvector):
+    evolver = pvector([1, 2]).evolver()
+    evolver[1] = 3
+
+    assert [evolver[i] for i in range(len(evolver))] == [1, 3]
+
+    del evolver[0]
+
+    assert evolver.persistent() == pvector([3])
+
+
+def test_compare_with_list(pvector):
+    v = pvector([1, 2, 3])
+
+    assert v == [1, 2, 3]
+    assert v != [1, 2]
+    assert v > [1, 2]
+    assert v < [2, 2]
+    assert [1, 2] < v
+    assert v <= [1, 2, 3]
+    assert v <= [1, 2, 4]
+    assert v >= [1, 2, 3]
+    assert v >= [1, 2]
+
+
+def test_python_no_c_extension_with_environment_variable():
+    from six.moves import reload_module
+    import pyrsistent._pvector
+    import pyrsistent
+    import os
+
+    os.environ['PYRSISTENT_NO_C_EXTENSION'] = 'TRUE'
+
+    reload_module(pyrsistent._pvector)
+    reload_module(pyrsistent)
+
+    assert type(pyrsistent.pvector()) is pyrsistent._pvector.PythonPVector
+
+    del os.environ['PYRSISTENT_NO_C_EXTENSION']
+
+    reload_module(pyrsistent._pvector)
+    reload_module(pyrsistent)
+
+
+def test_supports_weakref(pvector):
+    import weakref
+    weakref.ref(pvector())
+
+
+def test_failing_repr(pvector):
+    # See https://github.com/tobgu/pyrsistent/issues/84
+    class A(object):
+        def __repr__(self):
+            raise ValueError('oh no!')
+
+    with pytest.raises(ValueError):
+        repr(pvector([A()]))
